
/*!
 * async-util
 * JavaScript async utilities library
 *
 * Copyright (c) 2011 Enrico Marino
 * MIT license
 */

!function (exports) {

  exports.async = {};

  /**
   * Library version
   */
  async.version = 0.0.1;

  /**
   * Apply 'iterator' to each item in 'array' in parallel
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callback
   */

  async.parallel_each = function (array, iterator, callback) {
    var completed = 0
      , len = array.length
      , i
      ;

    function complete (err) {
      if (err) {
        callback(err);
        callback = function () {};
        return;
      }
      completed += 1;
      if (completed === len) {
        callback();
        return;
      }
    }

    for (i = 0; i < len; i += 1) {
      if (i in array) {
        iterator(array[i], complete);
      }
    }
  };

  /**
   * Apply 'iterator' for each item in 'array' in series
   * and call 'callback' when done
   *
   * @param {Array} array array
   * @param {Function} iterator iterator
   * @param {Function} callback callback
   */

  async.series_each = function (array, iterator, callback) {

    var completed = 0
      , len = array.length
      ;

    function iterate () {
      function complete (err) {
        if (err) {
          callback(err);
          callback = function () {};
          return;
        }
        completed += 1;
        if (completed === len) {
          callback();
          return;
        }
        iterate();
      }

      iterator(array[completed], complete);
    }

    iterate();
  };

}(this);